# Copyright © 2024, UChicago Argonne, LLC
# BSD OPEN SOURCE LICENSE. Full license can be found in LICENSE
# This portion of the script initializes the plugin, making it known to QGIS.
import os
from pathlib import Path
import shutil
import subprocess
import sys
from os.path import abspath, dirname, join, isfile, isdir
import qgis.utils


def install_from_requirement_file(requirements_file, args):
    root = Path(__file__).parent / "packages"
    root.mkdir(exist_ok=True)
    if (root / requirements_file).exists():
        # Requirements installed, so we can leave
        return

    reqs_file = root.parent / requirements_file
    if not reqs_file.exists():
        return
    print(f"Installing extra requirements from {reqs_file}")
    command = f"python -m pip install -r {reqs_file} -t {root} {args}"
    lines = [command]
    with subprocess.Popen(
        command,
        shell=True,
        stdout=subprocess.PIPE,
        stdin=subprocess.DEVNULL,
        stderr=subprocess.STDOUT,
        universal_newlines=True,
    ) as proc:
        lines.extend(proc.stdout.readlines())
    print(lines)
    # Most of these packages would end up being doubled up with QGIS' own ones, so we remove them
    # cpuinfo is a Tables dependency, but it is breaking the plugin, so we need to remove it too
    # It is not clear whether we will need the functionalities that cpuinfo bring to Tables
    # but for now we are removing it
    pkgs_to_remove = ["numpy", "scipy", "pandas", "shapely", "cpuinfo", "geopandas", "pyproj"]
    print(f"Removing packaged libs in favor of QGIS version: {', '.join(pkgs_to_remove)}")
    for fldr in list(os.walk(root))[0][1]:
        fldr_path = join(root, fldr)
        if any(pkg in fldr.lower() for pkg in pkgs_to_remove) and isdir(fldr_path):
            shutil.rmtree(fldr_path)
            print(f"  - [DELETED] {fldr_path}")

    with open(root / requirements_file, "w"):
        pass


def install_requirements():
    install_from_requirement_file("install_requirements.txt", "--no-deps")
    install_from_requirement_file("polaris_studio_requirements.txt", "")
