# Copyright © 2024, UChicago Argonne, LLC
# BSD OPEN SOURCE LICENSE. Full license can be found in LICENSE
import logging
from os.path import dirname, join

import pandas as pd
from polaris.runs.scenario_compression import ScenarioCompression
from polaris.utils.database.db_utils import read_and_close
from qgis.PyQt import QtWidgets, uic

from ..common_tools.path_drawer import PathDrawer

FORM_CLASS, _ = uic.loadUiType(join(dirname(__file__), "forms/show_path.ui"))

logger = logging.getLogger("polaris")


class PathDemandDB(QtWidgets.QDialog, FORM_CLASS):
    def __init__(self, qgis_project) -> None:
        QtWidgets.QDialog.__init__(self)
        self.iface = qgis_project.iface
        self.qgis_project = qgis_project
        self.setupUi(self)
        self.field_types = {}
        self.link_layer = self.qgis_project.layers["link"][0]
        self.path_drawer = PathDrawer({"Links": ["link", self.link_layer]})

        self.path = []

        self.do_path.clicked.connect(self.build_path)

    def build_path(self):
        path_id = str(self.path_id.text())
        if not path_id.isdigit() or len(path_id) < 1:
            self.qgis_project.show_error_message("Path entered is not numeric")
            return

        with read_and_close(ScenarioCompression.maybe_extract(self.qgis_project.demand_path)) as conn:
            df = pd.read_sql(f"Select value_link as link from Path_links where object_id={int(path_id)}", conn)
            if not df.shape[0]:
                self.qgis_project.show_error_message(f"Could not find a path with id {path_id}")
                return
            self.path = df.link.tolist()

        if self.rdo_selection.isChecked():
            self.path_drawer.create_path_with_selection(self.path)
        else:
            _ = self.path_drawer.create_traffic_path_with_scratch_layer(
                path=self.path, layer_name=f"Path ID-{self.path_id.text()}"
            )

    def exit_procedure(self):
        self.close()
