# Copyright © 2024, UChicago Argonne, LLC
# BSD OPEN SOURCE LICENSE. Full license can be found in LICENSE
from os.path import dirname, join

from qgis.PyQt import uic
from qgis.PyQt.QtWidgets import QDialog

FORM_CLASS, _ = uic.loadUiType(join(dirname(__file__), "forms/geo_consistency.ui"))


class GlobalGeoconsistency(QDialog, FORM_CLASS):
    def __init__(self, _PQgis):
        QDialog.__init__(self)
        self.iface = _PQgis.iface
        self.setupUi(self)

        self._PQgis = _PQgis
        self._p = _PQgis.network
        self.worker_thread = self._p.geo_consistency
        self.__mt = ""

        self.but_process.clicked.connect(self.process_consistency)
        self.but_clear.clicked.connect(self.clear_all_actions)
        self.but_select_all.clicked.connect(self.select_all_actions)
        self.progress_box.setVisible(False)
        self.progress_box.setEnabled(False)
        self.setFixedHeight(232)
        self.__chk_boxes = [
            self.chk_active_links,
            self.chk_links,
            self.chk_zone_association,
            self.chk_location_association,
        ]
        self.items = [self.but_process, self.but_clear, self.but_select_all, self.progress_box, self.options_box]

    def clear_all_actions(self):
        for item in self.__chk_boxes:
            item.setChecked(False)

    def select_all_actions(self):
        for item in self.__chk_boxes:
            item.setChecked(True)

    def process_consistency(self):
        for item in self.items:
            item.setVisible(not item.isVisible())
            item.setEnabled(not item.isEnabled())
        self.setFixedHeight(176)

        self.worker_thread.set_do_active_network_association(self.chk_active_links.isChecked())
        self.worker_thread.set_do_network_association(self.chk_links.isChecked())
        self.worker_thread.set_do_zone_association(self.chk_zone_association.isChecked())
        self.worker_thread.set_do_location_association(self.chk_location_association.isChecked())

        self.worker_thread.geoconsistency.connect(self.signal_handler)
        self.worker_thread.start()
        self.exec_()

    def signal_handler(self, val):
        if len(val) == 1:
            if val[0] == "finished_geoconsistency_procedure":
                self.close()
            return

        bar = self.progressBar if val[1] == "master" else self.progressBar2
        lbl = self.lbl_progress if val[1] == "master" else self.lbl_progress2

        if val[0] == "start":
            lbl.setText(val[3])
            bar.setRange(0, val[2])
            bar.setValue(0)
            if val[1] != "master":
                self.__mt = val[3]
        elif val[0] == "update":
            bar.setValue(val[2])
            if len(val) == 4 and bar.maximum() == val[2]:
                self.progressBar.setValue(self.progressBar.value() + 1)
